@@ -1,25 +1,27 @@
 # -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+# ***** BEGIN LICENSE BLOCK *****
 # Version: MPL 1.1/GPL 2.0/LGPL 2.1
-# 
+#
 # The contents of this file are subject to the Mozilla Public License Version
 # 1.1 (the "License"); you may not use this file except in compliance with
 # the License. You may obtain a copy of the License at
 # http://www.mozilla.org/MPL/
-# 
+#
 # Software distributed under the License is distributed on an "AS IS" basis,
 # WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 # for the specific language governing rights and limitations under the
 # License.
-# 
+#
 # The Original Code is the Thunderbird Preferences System.
-# 
-# The Initial Developer of the Original Code is Scott MacGregor.
+#
+# The Initial Developer of the Original Code is
+# Scott MacGregor.
 # Portions created by the Initial Developer are Copyright (C) 2005
 # the Initial Developer. All Rights Reserved.
-# 
+#
 # Contributor(s):
 #   Scott MacGregor <mscott@mozilla.org>
-# 
+#
 # Alternatively, the contents of this file may be used under the terms of
 # either the GNU General Public License Version 2 or later (the "GPL"), or
 # the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
@@ -31,7 +33,7 @@
 # and other provisions required by the GPL or the LGPL. If you do not delete
 # the provisions above, a recipient may use your version of this file under
 # the terms of any one of the MPL, the GPL or the LGPL.
-# 
+#
 # ***** END LICENSE BLOCK *****
 
 var gAdvancedPane = {
@@ -200,5 +202,58 @@ var gAdvancedPane = {
     textbox.disabled = !document.getElementById('markAsRead').checked;
     if (!textbox.disabled && aFocusTextBox)
         textbox.focus();
+  },
+  
+  /** 
+   * open the return receipts configuration dialog 
+   */
+  showReturnReceipts: function()
+  {
+    document.documentElement.openSubDialog("chrome://messenger/content/preferences/receipts.xul",
+                                           "", null);
+  },  
+  
+  /** 
+   * open the connections dialog 
+   */
+  showConnections: function ()
+  {
+    document.documentElement
+            .openSubDialog("chrome://messenger/content/preferences/connection.xul",
+                           "", null);
+  },
+  
+  /** 
+   * open the offline settings dialog
+   */
+  showOffline: function()
+  {
+    document.documentElement
+            .openSubDialog("chrome://messenger/content/preferences/offline.xul",
+                           "", null);  
+  },
+  
+  showCertificates: function ()
+  {
+    document.documentElement.openWindow("mozilla:certmanager", "chrome://pippki/content/certManager.xul",
+                                        "width=600,height=400", null);
+  },
+  
+  showCRLs: function ()
+  {
+    document.documentElement.openWindow("Mozilla:CRLManager", "chrome://pippki/content/crlManager.xul",
+                                        "width=600,height=400", null);
+  },
+  
+  showOCSP: function ()
+  {
+    document.documentElement.openSubDialog("chrome://mozapps/content/preferences/ocsp.xul",
+                                           "", null);
+  },
+  
+  showSecurityDevices: function ()
+  {
+    document.documentElement.openWindow("mozilla:devicemanager", "chrome://pippki/content/device_manager.xul",
+                                        "width=600,height=400", null);
   }
 };
